<?xml version="1.0" encoding="utf-8" ?>
<xsl:stylesheet
  version="2.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:xs="http://www.w3.org/2001/XMLSchema"
  xmlns:saxon="http://saxon.sf.net/"
  xmlns:my="myFunctions"
  exclude-result-prefixes="saxon my xs" 
>
<!-- 
<purpose> generate html view (probably various formats of a MDRepository response.</purpose>
<params>
<param name=""></param>
</params>
<history>	
	<change on="2010-03-26" type="created" by="vr">based on cmdi/scripts/cmdinst2view.xsl</change>	
</history>
-->
<xsl:import href="model2matrix.xsl"/>
<xsl:import href="terms2view.xsl"/>

<!--  method="xhtml" is saxon-specific! prevents  collapsing empty <script> tags, that makes browsers choke -->
<xsl:output method="xhtml" media-type="application/xhtml+xml" indent="yes" encoding="UTF-8" 
doctype-public="-//W3C//DTD XHTML 1.0 Transitional//EN" /> 

  <!--
  if this is added  to the output-element, it terribly slows down requests with recordset being injected via document() function
   (like: http://localhost:8080/MDService2/docs/htmlpagetest/queries2)
  doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd" />  
  --> 

<xsl:include href="cmd_commons.xsl"/>

<!-- <xsl:param name="format" select="'table'" /> table|list|detail -->
<xsl:param name="cols" >Id,Title,Location
<!-- 
<col label="id">Id</col>
<col label="id">idno</col>
<col label="name">Name</col>
<col label="title">Title</col>
<col label="title">title</col>
<col label="loc" >Location</col>
<col label="loc" >pubPlace</col>
<col label="lang" >language</col>

 -->
</xsl:param>

<xsl:param name="mode" select="'html'" />
<xsl:param name="title" >
	<xsl:text>MDRecordSet </xsl:text>
</xsl:param>

<xsl:variable name="columns" >
<!-- <xsl:message>COLUMNS: <xsl:value-of select="$cols" /><xsl:value-of select="string-length($cols)" /></xsl:message>  -->
	<xsl:variable name="colarray" select="tokenize($cols,',')"/>
	<xsl:for-each select="$colarray" >
		<col key="{.}" label="{replace(.,'^.*\.([^.]+)$','$1')}" xpath_osbolete="{replace(.,'\.','/')}" xpath="{my:index2xpath(.)}" />
		<!-- if (contains(.,'.')) then reverse(.) else . -->
	</xsl:for-each>
	<xsl:if test="count($colarray)=0" >	
		<xsl:copy-of select="$autocolumns"></xsl:copy-of>
		<!-- <xsl:message>AUTOCOLUMNS: <xsl:copy-of select="$autocolumns" /><xsl:value-of select="count($autocolumns)" /></xsl:message> -->
	</xsl:if>
	
</xsl:variable>

<xsl:variable name="autocolumns" >
	<xsl:variable name="precompute">
		<xsl:for-each-group select="//Components//*[not(*)][exists(text())]" group-by="name()" >			
			<xsl:sort select="sum(current-group()/string-length(string-join(text(),';')))" order="descending" />
			<!-- <xsl:message>precomputing: <xsl:value-of select="name()" />
				count: <xsl:value-of select="count(current-group())" /> sum:<xsl:value-of select="sum(current-group()/string-length(string-join(text(),';')))" /></xsl:message> -->
			<col key="{name()}" label="{name()}" xpath="{name()}" />
		</xsl:for-each-group>
	</xsl:variable>
	 <!-- <xsl:message>precompute: <xsl:copy-of select="$precompute" /><xsl:value-of select="count($precompute)" /></xsl:message>-->
	<xsl:copy-of select="$precompute/*[position() &lt; $autocolumns_limit]"></xsl:copy-of> 
</xsl:variable>

<xsl:template name="continue-root" >

<xsl:for-each select="searchRetrieveResponse" >
<div>
	<xsl:if test="not(contains($format, 'htmlsimple'))">
		<xsl:call-template name="header" />
	</xsl:if>
 	<xsl:call-template name="query-input" />
	
	<xsl:choose>
				<xsl:when test="contains($format,'htmltable')">
					<xsl:apply-templates select="records" mode="table" />
				</xsl:when>				
				<xsl:when test="contains($format,'htmllist')">
					<xsl:apply-templates select="records" mode="list" />
				</xsl:when>				
				<xsl:when test="contains($format, 'htmldetail')">
					<xsl:apply-templates select="records" mode="detail" />
				</xsl:when>
				<xsl:when test="contains($format, 'htmlpage')">
					<xsl:apply-templates select="records" mode="table" />
				</xsl:when>
				<xsl:when test="contains($format, 'htmlpagelist')">
					<xsl:apply-templates select="records" mode="list" />
				</xsl:when>		
				<xsl:when test="contains($format, 'htmlsimple')">
					<xsl:apply-templates select="records" mode="simple" />
				</xsl:when>		
				<xsl:otherwise>mdset2view: unrecognized format: <xsl:value-of select="$format" /></xsl:otherwise>
	</xsl:choose>
	</div>
</xsl:for-each>	
	
</xsl:template>	
	
	
<xsl:template match="*" >
	<!-- <xsl:param name="lv" select="1" />-->
	<!--<xsl:variable name="lv" select="count(ancestor::d)" /> -->
	<!-- <xsl:if test="string-length(string-join(.//text(),'')) &gt; 1" > -->		 
		<xsl:variable name="has_text" >
			<xsl:choose>
			<!-- FIXME: text()[1] ? is that safe?-->
				<xsl:when test="normalize-space(text()[1])='Unspecified'">unspecified</xsl:when>
				<xsl:when test="not(normalize-space(text()[1])='')">text</xsl:when>				
				<xsl:otherwise>empty</xsl:otherwise>
			</xsl:choose>
			
		</xsl:variable>
		<div class="elem {$has_text}" ><span class="elem_name"><xsl:value-of select="name()" />:</span>
		<span class="elem_value"><xsl:value-of select="text()" /></span>
			<xsl:apply-templates select="*" />					
		</div>
	
</xsl:template>	
	
	<!--
sample header:	
<numberOfRecords>524</numberOfRecords>
    <echoedSearchRetrieveRequest>//Title[contains(.,'an')] /db/cmdi-mirror/silang_data 1 50</echoedSearchRetrieveRequest>
    <diagnostics>50</diagnostics>
    -->
<xsl:template name="header">	
	<div class="result-header" >
	<xsl:attribute name="max_value"><xsl:value-of select="numberOfRecords" /></xsl:attribute>
	<xsl:variable name="cnt_hits" select="number(diagnostics)" />
	
								
	<form action="{my:formURL('search', 'htmlpage', $q)}" method="get">
			
		<span class="label" >hits: </span><span class="value hilight"><xsl:value-of select="numberOfRecords" /></span>;
		<span class="label" >from:</span><span>
		<input type="text" name="startRecord" class="value start_record paging-input">
			<xsl:attribute name="value">
    		<xsl:value-of select="$startRecord"/>
    		</xsl:attribute>
		</input>
		</span>
	
		<span class="label" >max:</span><span><input type="text" name="maximumRecords" class="value maximum_records paging-input" ><xsl:attribute name="value">
    		<xsl:value-of select="if(number($cnt_hits) &lt; number($maximumRecords)) then $cnt_hits else $maximumRecords"/>    
  		</xsl:attribute></input></span> 
  		
  		<xsl:if test="$repository_name != ''">
  			<input type="hidden" name="repository"><xsl:attribute name="value">
    			<xsl:value-of select="$repository_name"/>
  				</xsl:attribute>
			</input>
  		</xsl:if>
  	 
  	 	<input type="hidden" name="query"><xsl:attribute name="value">
    			<xsl:value-of select="$q"/>
  				</xsl:attribute>
		</input>
			
		 <input type="submit" value="" class="cmd cmd_reload">
		</input>
	 <!--  number($startRecord - $pageRecordCount)<a href="{my:formsearchURL($q,'1','20',$repository_name)}"><span class="cmd cmd_reload" /></a> -->
	 	<xsl:variable name="prev_startRecord">
	 		<xsl:choose>
	 			<xsl:when test="number($startRecord) - number($maximumRecords) > 0"> <xsl:value-of select="format-number(number($startRecord) - number($maximumRecords),'#')"></xsl:value-of></xsl:when>
	 			<xsl:otherwise><xsl:value-of select="1"></xsl:value-of></xsl:otherwise>
	 		</xsl:choose>
	 	</xsl:variable>
	 	<xsl:variable name="next_startRecord">
	 		<xsl:choose>
	 			<xsl:when test="number($startRecord) + number($maximumRecords) > number(numberOfRecords)"> <xsl:value-of select="$startRecord"></xsl:value-of></xsl:when>
	 			<xsl:otherwise><xsl:value-of select="format-number(number($startRecord) + number($maximumRecords),'#')"></xsl:value-of></xsl:otherwise>
	 		</xsl:choose>
	 	</xsl:variable>
	 	<a href="{my:formsearchURL($q,$prev_startRecord,$maximumRecords,$repository_name)}">
			<span>	
			<xsl:choose>
			<xsl:when test="$startRecord = '1'">
				<xsl:attribute name="class"><xsl:value-of select="'cmd cmd_prev disabled'"></xsl:value-of></xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class"><xsl:value-of select="'cmd cmd_prev '"></xsl:value-of></xsl:attribute>
			</xsl:otherwise>
			</xsl:choose>
			</span>
		</a>
		<a href="{my:formsearchURL($q,$next_startRecord,$maximumRecords,$repository_name)}">
			<span class="cmd cmd_next">	
			<xsl:choose>
			<xsl:when test="number($startRecord) + number($maximumRecords) >= number(numberOfRecords)">
				<xsl:attribute name="class"><xsl:value-of select="'cmd cmd_next disabled'"></xsl:value-of></xsl:attribute>
			</xsl:when>
			<xsl:otherwise>
				<xsl:attribute name="class"><xsl:value-of select="'cmd cmd_next '"></xsl:value-of></xsl:attribute>
			</xsl:otherwise>
			</xsl:choose>
			</span>
		</a>
	<div>
  		  	<span class="label" >format:</span>
			<select class="value-format" name="format">
			
			<option value="htmltable">
				<xsl:choose>
				<xsl:when test="contains($format, 'htmltable')">
					<xsl:attribute name="selected">selected</xsl:attribute>
				</xsl:when>
				</xsl:choose><xsl:text>table</xsl:text>		
			</option>
			<option value="htmllist">
				<xsl:choose>
				<xsl:when test="contains($format, 'htmllist')">
					<xsl:attribute name="selected">selected</xsl:attribute>
				</xsl:when>
				</xsl:choose><xsl:text>list</xsl:text>
			</option>
			</select>
		</div>
		</form>
	<span class="cmd cmd_detail" />	
		<div class='ui-widget-content columns-wrapper'>
			<div class='cmds'><span class="cmd cmd_reload" /><span class="cmd cmd_save" /><span class="cmd cmd_savenew" /></div>
			<div class='query-columns'>
			<label>Columns</label>
				<xsl:call-template name="columns" />
			</div>
		</div>	
		
    </div>

    <div class="note" ><span class="label" >echo: </span><span class="value"><xsl:value-of select="echoedSearchRetrieveRequest" /></span>;<span class="label" >duration: </span><span class="value"><xsl:value-of select="extraResponseData/duration" /></span>;</div>
    	
</xsl:template>	

<xsl:template name="columns">
	<table>
	<tr><td valign="top">
	<div class="columns-selected" >	
	<!-- 
		<span class="label">selected columns</span>
		<ul class="query-columnslist">
			<xsl:for-each select="$columns/*">
  				<li><xsl:value-of select="@label"/><span class='cmd cmd_del'/></li>
  			</xsl:for-each>
  		</ul>
	 -->
  	</div>
  	</td><td valign="top">
  	<!-- 
  		<div>
  		  	<span class="label" >format:</span>
			<select class="value-format">
			
			<option value="htmltable">
				<xsl:choose>
				<xsl:when test="contains($format, 'htmltable')">
					<xsl:attribute name="selected">selected</xsl:attribute>
				</xsl:when>
				</xsl:choose><xsl:text>table</xsl:text>		
			</option>
			<option value="htmllist">
				<xsl:choose>
				<xsl:when test="contains($format, 'htmllist')">
					<xsl:attribute name="selected">selected</xsl:attribute>
				</xsl:when>
				</xsl:choose><xsl:text>list</xsl:text>
			</option>
			</select>
		</div>
  	 -->
  	
  	<xsl:if test="extraResponseData/usedProfiles" >
  		<div class="used-profiles">
	  		<span class="label">used profiles in whole result</span>
			<ul>
				<xsl:for-each select="extraResponseData/usedProfiles/profile" >
					<li><a href="{@id}" ><xsl:value-of select="@name"/></a><span class="note">|<xsl:value-of select="@count"/>|</span></li>
				</xsl:for-each>
			</ul>
		</div>
	</xsl:if>
	</td></tr>	
	<xsl:variable name="resultSummary" >		
		<!-- wrap in <Terms> to make digestible for model2matrix -->
		<xsl:variable name="terms" >
			<Terms>
			<!-- TODO: if already provided by mdrepo, use that, otherwise DIY -->		
			<xsl:choose>
				<xsl:when test="extraResponseData/resultSummary"><xsl:copy-of select="extraResponseData/resultSummary/Term" /></xsl:when>
				<xsl:otherwise>
				<!-- TODO! -->
				</xsl:otherwise>
			</xsl:choose>
			</Terms>
		</xsl:variable>
		<xsl:apply-templates select="$terms" mode="matrix" />
	</xsl:variable> 	
	<!--  <xsl:message><xsl:value-of select="$resultSummary/*/name()" /> </xsl:message>-->
	
	<tr><td colspan="2">	
	<div class="result-summary">	
		<span class="label">structure summary of shown result</span>	
		<!--  calling template in terms2view.xsl -->
		<!--  <xsl:apply-templates select="$resultSummary" mode="terms-tree" />-->		
	</div>	
	</td></tr>
	</table>
</xsl:template>

<xsl:template match="records" mode="table">
    
	<table class="show">		
		<thead><tr><th>pos</th>		
			<xsl:for-each select="$columns/*">
  				<th title="{@key}" ><xsl:value-of select="@label"/></th>
  			</xsl:for-each>  			
		</tr></thead>
		<tbody>		
			<xsl:apply-templates select="CMD" mode="table" />			
		</tbody>
	</table>

</xsl:template>


<xsl:template match="records" mode="list">
    
	<table class="show">		
		<thead><tr><th>pos</th><th>record</th>			  			
		</tr></thead>
		<tbody>		
			<xsl:apply-templates select="CMD" mode="list" />			
		</tbody>
	</table>

</xsl:template>

<!-- FIXME:   there should be a @match=`record` template to make it SRU-compatible -->

<xsl:template match="CMD" mode="table">

	<xsl:variable name="curr_record" select="." />
	<xsl:variable name="fields" >
		<xsl:for-each select="$columns/*">
	  			<!-- <td ><xsl:value-of select="$curr_cmd//*[name()=current()]" /></td> -->
	<!--  	<xsl:message><xsl:copy-of select="."></xsl:copy-of> </xsl:message> -->
	 				 <xsl:variable name="curr_col_xpath" select="@xpath" />
	  			<td> 			
	  				<xsl:for-each select="$curr_record">
	  					<xsl:call-template name="format-field" >
	  						<xsl:with-param name="elems" select="saxon:evaluate(concat('.//', $curr_col_xpath))" />
	  					</xsl:call-template> 					
	  				
	  				</xsl:for-each>
	  			</td>
	  			</xsl:for-each>
	  </xsl:variable>

	<xsl:call-template name="record-table-row" >
		<xsl:with-param name="fields" select="$fields" />
	</xsl:call-template>
	
</xsl:template>

<xsl:template match="CMD" mode="list">

		<xsl:variable name="curr_record" select="." />			
		<xsl:variable name="cmd_sel_fields" >
			<xsl:for-each select="$columns/*">
 				<xsl:variable name="curr_col_xpath" select="@xpath" />  			 			
  				<xsl:for-each select="$curr_record">  				
  					<xsl:copy-of select="saxon:evaluate(concat('.//', $curr_col_xpath))" />
  				</xsl:for-each>  			
  			</xsl:for-each>
  		</xsl:variable>
		<xsl:variable name="fields" >
			<div>
				<xsl:apply-templates select="$cmd_sel_fields" mode="format-xmlelem" />
			</div>
		</xsl:variable>

	<xsl:call-template name="record-table-row" >
		<xsl:with-param name="fields" select="$fields" />
	</xsl:call-template>
											
</xsl:template>


<xsl:template name="record-table-row">
<xsl:param name="fields" />
<!-- @field absolute_position compute records position over whole recordset, ie add `startRecord` (important when paging)
 -->
	<xsl:variable name="absolute_position" select="if (number($startRecord)=number($startRecord)) then number($startRecord) + position() - 1 else  position()" />
	
		<tr>
		<td rowspan="2">
			<xsl:choose>
				<xsl:when test="exists(.//MdSelfLink)" >
					<xsl:choose>
						<xsl:when test="contains($format, 'htmlsimple')" >
						<a class="internal" href="{concat('../../record/htmldetail/', my:encodePID(.//MdSelfLink), '?repository=', $repository_name)}" >
								<xsl:value-of select="$absolute_position" />
						</a>						
						</xsl:when>
						<xsl:otherwise>
						<a class="internal" href="{concat('record/htmldetail/', my:encodePID(.//MdSelfLink), '?repository=', $repository_name)}" >
								<xsl:value-of select="$absolute_position" />
						</a>
						</xsl:otherwise>
					</xsl:choose>
					<span class="cmd cmd_detail" ></span>				
					<span class="cmd cmd_save" ></span>						
				</xsl:when>
				<xsl:otherwise><xsl:value-of select="$absolute_position" /></xsl:otherwise>
			</xsl:choose>
		</td>		
		<td colspan="{count($columns/*)}" >
				<xsl:call-template name="getContext" />
			<div class="title" ><xsl:call-template name="getName" /></div>			
		</td>
		</tr>
		<tr>
			<xsl:copy-of select="$fields" />						
		</tr>									
</xsl:template>


<xsl:template match="records" mode="simple">
    
    <form action="" method="get">
				<input type="text" name="q" ><xsl:attribute name="value"><xsl:value-of select="$q"/></xsl:attribute> </input>				
				<input type="submit" value="Submit" > </input><br/>				
	</form>
	<xsl:call-template name="header" />	
	<table class="show">		
		<thead><tr><th>pos</th>		
			<xsl:for-each select="$columns/*">
  				<th title="{@key}" ><xsl:value-of select="@label"/></th>
  			</xsl:for-each>  			
		</tr></thead>
		<tbody>	
			<xsl:apply-templates select="CMD" mode="table" />	 			
		</tbody>
	</table>

</xsl:template>

</xsl:stylesheet>